@@ -52,6 +52,7 @@ class OneLogin_Saml2_Auth(object):
         self.__settings = OneLogin_Saml2_Settings(old_settings, custom_base_path)
         self.__attributes = []
         self.__nameid = None
+        self.__nameid_format = None
         self.__session_index = None
         self.__session_expiration = None
         self.__authenticated = False
@@ -97,6 +98,7 @@ class OneLogin_Saml2_Auth(object):
             if response.is_valid(self.__request_data, request_id):
                 self.__attributes = response.get_attributes()
                 self.__nameid = response.get_nameid()
+                self.__nameid_format = response.get_nameid_format()
                 self.__session_index = response.get_session_index()
                 self.__session_expiration = response.get_session_not_on_or_after()
                 self.__authenticated = True
@@ -214,6 +216,15 @@ class OneLogin_Saml2_Auth(object):
         """
         return self.__nameid
 
+    def get_nameid_format(self):
+        """
+        Returns the nameID Format.
+
+        :returns: NameID Format
+        :rtype: string|None
+        """
+        return self.__nameid_format
+
     def get_session_index(self):
         """
         Returns the SessionIndex from the AuthnStatement.
@@ -307,7 +318,7 @@ class OneLogin_Saml2_Auth(object):
             parameters['Signature'] = self.build_request_signature(saml_request, parameters['RelayState'], security['signatureAlgorithm'])
         return self.redirect_to(self.get_sso_url(), parameters)
 
-    def logout(self, return_to=None, name_id=None, session_index=None, nq=None):
+    def logout(self, return_to=None, name_id=None, session_index=None, nq=None, name_id_format=None):
         """
         Initiates the SLO process.
 
@@ -323,6 +334,9 @@ class OneLogin_Saml2_Auth(object):
         :param nq: IDP Name Qualifier
         :type: string
 
+        :param name_id_format: The NameID Format that will be set in the LogoutRequest.
+        :type: string
+
         :returns: Redirection url
         """
         slo_url = self.get_slo_url()
@@ -334,12 +348,15 @@ class OneLogin_Saml2_Auth(object):
 
         if name_id is None and self.__nameid is not None:
             name_id = self.__nameid
+        if name_id_format is None and self.__nameid_format is not None:
+            name_id_format = self.__nameid_format
 
         logout_request = OneLogin_Saml2_Logout_Request(
             self.__settings,
             name_id=name_id,
             session_index=session_index,
-            nq=nq
+            nq=nq,
+            name_id_format=name_id_format
         )
         self.__last_request = logout_request.get_xml()
         self.__last_request_id = logout_request.id
